/*
    Valtra Tractors
    Copyright 2004 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __SPRITES_H
#define __SPRITES_H

#include <vector>
#include <SDL.h>

using std::vector;

class Sprite {
private:
public:
	SDL_Surface *img;
	bool update;
	SDL_Rect updateRect; // relative rect to be redrawn
	SDL_Rect pos; // absolute rect
	int id;
	bool old;
	bool relative;
	bool multUpdate;
	Sprite(SDL_Surface *s, const SDL_Rect &p, bool u); // if u==true && s==NULL, p must have w and h set
	Sprite(SDL_Surface *s, const SDL_Rect &p, const SDL_Rect &u); // if u.x>0 || u.y > 0, remember that p points to the real origin of the img
	Sprite(SDL_Surface *s, const SDL_Rect &p); // a big sprite with no updateRect, use redrawRect()
	~Sprite() {}
};

class SpriteEngine {
private:
	int nextId;
	vector<Sprite*> *curSprites;
	vector<Sprite*> *lastSprites;
	vector<SDL_Rect> rects;
	vector<SDL_Rect> lastRects;
	void (*updatedRect)(const SDL_Rect &r);
	void deleteVector(vector<Sprite*> *vect);
	bool rectInside(const SDL_Rect &a, const SDL_Rect &b);
	bool rectsIntersect(const SDL_Rect &a, const SDL_Rect &b, SDL_Rect &out);
	bool rectsOverlapMuch(const SDL_Rect &a, const SDL_Rect &b);
	void mergeRects(const SDL_Rect &a, const SDL_Rect &b, SDL_Rect &out);
	void mergeRects(bool all = false);
	void mergeRects(vector<SDL_Rect> &list, bool all = false);
	int countSides(const SDL_Rect &a, const SDL_Rect &b);
	void splitOverlap(vector<SDL_Rect> &list);
	void removeInsideRects();
public:
	SpriteEngine();
	~SpriteEngine();
	int getNextId();
	void addSprite(int id, Sprite *sprite);
	void redrawRect(const SDL_Rect &r);
	void redrawAll();
	void calcUpdates();
	void update(SDL_Surface *screen, SDL_Surface *bkg);
	void setUpdatedRect(void (*func)(const SDL_Rect &r));
	Sprite *getSprite(int id);
	void reset();
	void getRects(vector<SDL_Rect> &list);
};


#endif
